package com.ads;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.Map.Entry;
import java.util.Set;

import com.helper.IRServiceHelper;
import com.helper.PropertiesUtil;

public class ContentBasedAds {
	
	long[][] queryArray;
	long[][] docVectorArray;
	long[][] adsVectorArray;
	long[][] adInputVector;
	long termCount;
	long wordsCount;
	IRServiceHelper irServiceHelper;
	Map<Long,String> documentList;
	Map<Long,String> queryList;
	Map<Long,String> adsInputList;
	
	PropertiesUtil properties;
	
	public ContentBasedAds()
	{
		irServiceHelper = new IRServiceHelper();
		documentList = readLines(properties.getProperty("adserver.irsystem.documents.urls.file"));
		queryList = readLines(properties.getProperty("adserver.irsystem.documents.adlist.file"));
	}
	
	public void init()
	{
		termCount=IRServiceHelper.countLines(properties.getProperty("adserver.irsystem.bagofwords.file"));
		wordsCount=IRServiceHelper.countLines(properties.getProperty("adserver.irsystem.documents.adlist.file"));
	}
	
	public void findCosine(String query)
	{
		String[] breakQuery = query.split(" ");		
		Map<Long,Double> consineResult = new HashMap<Long,Double>();
		List<Long> docIds = new ArrayList<Long>(); 
		Map<Long,String> adjacentMap = new HashMap<Long, String>();
		
		if(breakQuery!=null)
		{
			init();
			buildqueryVector(breakQuery);
			 docVectorArray= getdocumentVectorFromfile(properties.getProperty("adserver.irsystem.documentvector.file"),termCount);
			// adsArray = getdocumentVectorFromfile("G:\\SJSU course\\fall 14\\CS 297\\bagOfWords\\att1\\ads_list.txt", wordsCount);
			 
			for(int j=0;j< queryArray.length; j++)
			{
			
			for(int i=0;i < docVectorArray.length;i++)
			{
				double result = (getDotProduct(queryArray[j], docVectorArray[i]))/calculateDistance(queryArray[j], docVectorArray[i]);
				System.out.println(" cosine similarity for given query with document "+ i+" :"+ result );
				consineResult.put(new Long(i), result);
			}
			}
			
			System.out.println("");
			for(Map.Entry<Long,Double> entry : entriesSortedByValues(consineResult)) {
				  System.out.println(documentList.get(entry.getKey()) + " => " + entry.getValue());
				  
				  if(entry.getValue()!=0)
				  {
					  docIds.add(entry.getKey());
				  }
				}
			System.out.println("");
			//System.out.println(" building vector from relevant documents");
			
			for (Map.Entry<Long, String> entry : documentList.entrySet())
			{
				if(docIds.contains(entry.getKey()))
				{
					adjacentMap = irServiceHelper.getAdjacentWords(entry.getKey(), entry.getValue(), query);
				}
			}
			
			for (Map.Entry<Long, String> entry : adjacentMap.entrySet())
			{
			    System.out.println(entry.getValue());
			}
			
			// get bagofWords and adsVector
			Map<String,Long> bagOfWords = irServiceHelper.getBagOfWords(properties.getProperty("adserver.irsystem.bagofwords.file"));
			adsVectorArray =  new long [(int) wordsCount][(int) termCount]; 
			adsVectorArray	= irServiceHelper.buildAdvector(termCount, bagOfWords,
					properties.getProperty("adserver.irsystem.documents.adlist.file"));
			
			adInputVector = new long[(int) adjacentMap.size()][(int)termCount]; 
			
			for (Map.Entry<Long, String> entry : adjacentMap.entrySet())
			{
				String adQuery = entry.getValue().trim();
				if(!adQuery.isEmpty()){
					String[] adQueryArr = adQuery.split(" ");
									
					for (String queryStr : adQueryArr)
					{
						if(bagOfWords.containsKey(queryStr.toLowerCase()))
						{
							adInputVector[entry.getKey().intValue()-1][bagOfWords.get(queryStr).intValue()-1] +=1;
						}
					}
				}
			}
			
			// Get relevant ads based on user's query		
			
			Map<Long,Double> relevantAds = new HashMap<Long, Double>();
			
			for(int i=0;i< adInputVector.length ;i++)
			{
				long[] adArray = new long[(int) termCount];
				for (int j=0; j<adInputVector[i].length;j++)
				{
					adArray[j] = adsVectorArray[i][j];
				}
				
				long[] adInputArray = new long[(int) termCount];
				for(int k=0 ; k < adsVectorArray[i].length ;k++)
				{
					adInputArray[k] = adInputVector[i][k];
				}
				
				// find cosine similarity between adArray and adInputArray
				double adResult=(getDotProduct(adArray,adInputArray))/(calculateDistance(adArray,adInputArray));
				relevantAds.put((long) i,adResult);
			}		
			System.out.println("");
			System.out.println(" Relevant Ads:");
			int count = 0;
			for(Map.Entry<Long,Double> entry : entriesSortedByValues(relevantAds)) {
				if(count > 10 || entry.getKey().equals(0L)){
					break;
				}
				count++;
				System.out.println(queryList.get(entry.getKey())+ " => " + entry.getValue());
			}
		}
	}
	
	public Map<Long,String> readLines(String filename)
	{
		long count=0;
		Map<Long,String> readLineList = new HashMap<Long, String>();
		try {
			BufferedReader br = new BufferedReader(new FileReader(new File(filename)));
			String line;
			while((line = br.readLine()) != null)
			{
				readLineList.put(count, line);
				count++;
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return readLineList;
	}
	
	public static double calculateDistance(long[] queryArray, long[] docArray)
    {
        double Sum = 0.0;
        for(int i=0;i<queryArray.length;i++) {
           Sum = Sum + Math.pow((queryArray[i]-docArray[i]),2.0);
        }
        return Math.sqrt(Sum);
    }
	
	public static double getDotProduct(long[] queryArray, long[] docArray)
	{
		double sum=0;
		for (int i = 0; i < queryArray.length; i++) {
            sum += queryArray[i] * docArray[i];    
        }
		return sum;
	}
	
	public void buildqueryVector(String[] breakQuery)
	{
		BufferedReader br;
		queryArray = new long[1][(int)termCount];		
		try
		{
			br= new BufferedReader(new FileReader(new File("G:\\SJSU course\\fall 14\\CS 297\\bagOfWords\\att2\\bagofwords1.txt")));
			int qCount=0;
			while(qCount < breakQuery.length)
			{
				String line;
				long lineCount=0, fCount=1;
				while ((line = br.readLine()) != null)
				{
					if(line.toLowerCase().contains(breakQuery[qCount].toLowerCase()))
					{
						queryArray[qCount][(int)lineCount]=fCount;						
					}
					//queryList.put(lineCount, line);
					lineCount++;					
				}
				qCount++;
			}
			br.close();
		}catch(Exception e)
		{
			e.printStackTrace();
		}
		
	}
	
	public long[][] getdocumentVectorFromfile(String filename, long count)
	{
		BufferedReader br;
		 long[][] docVectorArray = new long [10][(int)count] ;
		
		try{
			br= new BufferedReader(new FileReader(new File(filename)));
			String docLine;
			int i=0;
			while((docLine = br.readLine()) != null)
			{
				if(docLine.isEmpty())
				{
					continue;
				}
				String[] docLineArr = docLine.split(",");
				int j=0;
				while( j < docLineArr.length)
				{
					docVectorArray[i][j]=Long.parseLong(docLineArr[j]);
					//System.out.println("  " +docVectorArray[i][j]);
					j++;
				}
				System.out.println("Document vector created for document :" +i);
				//documentList.put((long)i, docLine);
				i++;
			}
			
		}catch(Exception e)
		{
			e.printStackTrace();
		}
		
		return docVectorArray;
	}
	

	public static void main(String[] args){
		
		System.out.println("Enter search query: ");
		Scanner in = new Scanner(System.in);
		String input = in.nextLine();
		
		ContentBasedAds ads = new ContentBasedAds();
		ads.findCosine(input);
		
	}
	static <K,V extends Comparable<? super V>> List<Map.Entry<K,V>> entriesSortedByValues(Map<K,V> map) {
		List<Entry<K,V>> sortedEntries = new ArrayList<Entry<K,V>>(map.entrySet());

	    Collections.sort(sortedEntries, 
	            new Comparator<Entry<K,V>>() {
	                @Override
	                public int compare(Entry<K,V> e1, Entry<K,V> e2) {
	                    return e2.getValue().compareTo(e1.getValue());
	                }
	            }
	    );

	    return sortedEntries;
	}
}
